PlotTableTriangles<-function(var,poscols=1,posrows=2,toptitle=NULL,
                             sig_var=NULL,col_sig='black',pch_sig=18,
                             cex_sig=1,
                             xaxelab=NULL, xlab=NULL, 
                             yaxelab=NULL,ylab=NULL,
                             legend=TRUE,lab_legend=NULL,
                             cexleg=1,col_legend=NULL,
                             fileout=NULL,
                             size_units = 'px', res = 100,
                             figure.width = 1,
                             col=NULL,brk=NULL,...){
  
  ########################################################################
  # represents an array with three dimensions ncol,nrow,ncat
  # as a triangled table
  #
  # Inputs: 
  # --------
  # var: array with three dimensions: rows, columns and categories
  # fileout: name of the file to save the plot
  # toptitle: title
  # sig_var: array with the same dimensions that var (TRUE/FALSE)
  # col_sig: color of the siginificance points
  # pch_sig: symbol for the significance
  # cex_sig: size of the symbols
  # xaxelab: label for the x ticks
  # yaxelab: label for the y ticks
  # xlab: label for the x axis
  # ylab: label for the y axis
  # legend: if a legend should be added (default TRUE)
  # cexleg: legend size
  # col_legend: color for the legend
  # fileout: A string of full directory path and file name indicating where 
  #     to save the plot. If not specified (default), a graphics device will pop up. 
  # size_units: A string indicating the units of the size of the device 
  #     (file or window) to plot in. Set 'px' as default. See ?Devices and the 
  #     creator function of the corresponding device.
  # res: A positive number indicating resolution of the device (file or window) 
  #     to plot in. See ?Devices and the creator function of the corresponding device.
  # col: color of the triangles
  # brk: breaks for the triangles
  #---------
  # Verónica Torralba
  # December 2017
  ########################################################################
  
  
  # Checking the dimensions
  vardim<-dim(var)
  nrow<-vardim[posrows]
  ncol<-vardim[poscols]
  poscat<-vardim[setdiff(1:length(vardim),c(posrows,poscols))]
  
  # If there is any filenames to store the graphics, process them
  # to select the right device 
  if (!is.null(fileout)) {
    deviceInfo <- s2dverification:::.SelectDevice(fileout = fileout,
                                                  width = 80 * ncol * figure.width, 
                                                  height = 80 * nrow,
                                                  units = size_units, res = res)
    saveToFile <- deviceInfo$fun
    fileout <- deviceInfo$files
  }
  
  # Open connection to graphical device
  if (!is.null(fileout)) {
    saveToFile(fileout)
  } else if (names(dev.cur()) == 'null device') {
    dev.new(units = size_units, res = res,
            width = 8 * figure.width, height = 5)
  }
  
  if (!is.null(sig_var)){
    if (!is.logical(sig_var)){
      stop('sig_var array should be provided as TRUE/FALSE')
    }
    if (all(dim(sig_var)!=dim(var))){
      stop('sig_var array should have the same dimensions that var')
    }
  }
  
  if (length(vardim)!=3){
    stop('var must be an array with three dimensions (4, cols, rows) ')
  }
  
  if (poscat!=4 && poscat!=2){
    stop('var must be an array with one of the dimensions being equal to 4 or 2')
  }
  
  # The dimensions are named 
  names(dim(var))[poscols]<-'cols'
  names(dim(var))[posrows]<-'rows'
  names(dim(var))[setdiff(1:length(vardim),c(posrows,poscols))]<-'cat'
  
  if (!is.null(sig_var)){
    names(dim(sig_var))<-names(dim(var))
  }
  
  # Checking what is available and generating missing information 
  if (!is.null(lab_legend)&&length(lab_legend)!=4){
    stop('The legend should contain 4 names')
  }
  
  if (is.null(xaxelab)){
    xaxelab=1:ncol
  }
  if (is.null(yaxelab)){
    yaxelab=1:nrow
  }
  if (is.null(brk)){
    brk<-seq(min(var),max(var),length.out = 9)
  }
  if (is.null(col)){
    col<-rev(brewer.pal(length(brk)-1,'RdBu'))
  }
  
  # The colours for each triangle/category are defined
  var_cat <- array(col[length(col)], dim = vardim)
  names(dim(var_cat)) <- names(dim(var))
  for (i in (length(col) - 1):1) {
    var_cat[var < brk[i + 1]] <- col[i]
  }
  
  if(legend){
    layout(matrix(c(1,2,1,3),2,2,byrow=T),widths =c(10,2),heights=c(10,2))
    par(mar=c(5,4,4,0))
    if(is.null(lab_legend)){
      lab_legend=1:4
    }
  }
  par(oma=c(1,1,1,1))
  plot(ncol,nrow,xlim = c(0,ncol),ylim=c(0,nrow), xaxs="i",yaxs='i',type="n", 
       xaxt="n", yaxt="n",ann=F,axes=F)
  
  box(col='black',lwd=1)
  
  if (! is.null(toptitle)){
    title(toptitle, cex=1.5)
  }
  
  if (!is.null(xlab)){
    mtext(side = 1, text = xlab, line = 4, cex=1.5)
  }
  if (!is.null(ylab)){
    mtext(side = 2, text = ylab, line = 2.5, cex=1.5)
  }
  
  axis(1, at=(1:ncol)-0.5, las=2, labels=xaxelab, cex.axis=1.5)
  axis(2, at=(1:nrow)-0.5, las=2, labels=yaxelab, cex.axis=1.5)
  
  
  #The triangles are plotted
  for(p in 1:ncol){
    for(l in 1:nrow){
      if (poscat==4){
        coord_triangl <- list(xs=list(c(p-1, p-0.5, p-1),c(p-1, p-0.5, p),c(p, p-0.5, p),c(p-1, p-0.5, p)),
                              ys=list( c(l-1, -0.5+l, l), c(l-1, -0.5+l, l-1),c(l-1, -0.5+l, l),c(l, -0.5+l, l)))
        
        coord_sig <- list(x=c(p-0.75,p-0.5,p-0.25,p-0.5),y=c(l-0.5,l-0.75,l-0.5,l-0.25))
      }
      
      if (poscat==2){
        coord_triangl<- list(xs=list(c(p-1, p, p),c(p-1, p, p-1)),
                             ys=list( c(l-1,l-1, l), c(l-1, l, l)))
        coord_sig<- list(x=c(p-(2/3),p-(1/3)),y=c(l-(1/3),l-(2/3)))
      }
      for (n in 1:poscat) {
        polygon(coord_triangl$xs[[n]],
                coord_triangl$ys[[n]],
                col = Subset(
                  var_cat,
                  along = c('cat', 'cols', 'rows'),
                  indices = list(n, p, l)))
        if (!is.null(sig_var) &&
            Subset(sig_var,along = c('cat', 'cols', 'rows'),
                   indices = list(n, p, l))) {
          points(
            x = coord_sig$x[n],
            y = coord_sig$y[n],
            pch = pch_sig,
            cex = cex_sig,
            col = col_sig
          )
        }
      }
    }
  }
  
  # legend
  
  if(legend){
    # Colorbar
    par(mar=c(0,0,0,0))
    ColorBar(brks = brk, cols = col, vert=T,draw_ticks = T, draw_separators = T,
             extra_margin = c(0,0,2.5,0),label_scale = 1.5,...)
    
    # legend in the bottom-corner
    if(is.null(col_legend)){
      col1<-'#2c7fb8'
    }else{
      col1<-col_legend
    }
    par(mar=c(0.5,2.5,0,2.5))
    plot(1,1,xlim = c(0,1),ylim=c(0,1), xaxs="i",yaxs='i',type="n", 
         xaxt="n", yaxt="n",ann=F,axes=F)
    
    box(col=col1)
    p=l=1
    if (poscat==4){
      coord_triangl <- list(xs=list(c(p-1, p-0.5, p-1),c(p-1, p-0.5, p),c(p, p-0.5, p),c(p-1, p-0.5, p)),
                            ys=list( c(l-1, -0.5+l, l), c(l-1, -0.5+l, l-1),c(l-1, -0.5+l, l),c(l, -0.5+l, l)))
      
      coord_sig <- list(x=c(p-0.75,p-0.5,p-0.25,p-0.5),y=c(l-0.5,l-0.75,l-0.5,l-0.25))
    }
    
    if (poscat==2){
      coord_triangl<- list(xs=list(c(p-1, p, p),c(p-1, p, p-1)),
                           ys=list( c(l-1,l-1, l), c(l-1, l, l)))
      coord_sig<- list(x=c(p-(2/3),p-(1/3)),y=c(l-(1/3),l-(2/3)))
    }
    for (n in 1:poscat) {
      polygon(coord_triangl$xs[[n]],
              coord_triangl$ys[[n]],border=col1)
      text(x=coord_sig$x[[n]],y=coord_sig$y[[n]],labels = lab_legend[n],cex=cexleg,col=col1)
      
    }
  }
  
  # If the graphic was saved to file, close the connection with the device
  if (!is.null(fileout)) dev.off()
}
  